OBJECT_CREATE = {
    "apiVersion": "objects.mavq.io/v1",
    "kind": "Operation",
    "metadata": {
        "tenant": "admin"
    },
    "spec": {
        "manifest": {
            "objectConfigurationSpec": {
                "name": "pythonobject",
                "display": {
                    "singularLabel": "pythonobject",
                    "pluralLabel": "pythonobject",
                    "description": ""
                },
                "visibility": "public-read-write-delete",
                "storage": {
                    "mode": "RDBMS"
                }
            },
            "systemFields": {
                "nameFieldDisplay": {
                    "label": "name",
                    "description": "",
                    "helpText": "",
                    "placeholder": ""
                },
                "nameFieldUnique": True
            }
        },
        "options": {
            "retryLimit": 5
        },
        "type": "create_object"
    }
}

OBJECT_UPDATE = {
    "kind": "ObjectConfiguration",
    "apiVersion": "objects.mavq.io/v1",
    "metadata": {
        "system": {
            "controls": {
                "access": {
                    "readOnly": False,
                    "hidden": False,
                    "nonDeletable": False
                }
            },
            "hooks": [],
            "audit": {
                "createdAt": "2022-06-07T08:36:00.183Z",
                "updatedAt": "2022-06-07T08:36:06.405Z",
                "createdBy": "mavq:core://tenants/admin/users/vishal-agarwal-back",
                "updatedBy": "mavq:core://tenants/admin/users/vishal-agarwal-back",
                "versions": [],
                "operations": [],
                "owner": "mavq:core://tenants/admin/users/vishal-agarwal-back",
                "versionNumber": 1
            },
            "selfLink": "mavq:objects://tenants/admin/object_configurations/pythonobject"
        },
        "tenant": "admin",
        "uid": "pythonobject"
    },
    "spec": {
        "name": "pythonobject",
        "display": {
            "singularLabel": "pythonobjectUpdate",
            "pluralLabel": "pythonobjectUpdate",
            "description": ""
        },
        "visibility": "public-read-write-delete",
        "storage": {
            "mode": "RDBMS",
            "versioning": {
                "versionLimit": 2,
                "versionDays": 30
            }
        }
    }
}

FIELD_CREATE = {
    "kind": "ObjectField",
    "apiVersion": "objects.mavq.io/v1",
    "metadata": {
        "tenant": "admin"
    },
    "spec": {
        "objectConfigurationUri": "mavq:objects://tenants/admin/object_configurations/abcprojects",
        "display": {
            "description": "",
            "helpText": "",
            "label": "emailfield",
            "placeholder": ""
        },
        "definition": {
            "type": "EMAIL",
            "name": "emailfield",
            "mandatory": True,
            "indexing": False,
            "immutable": False,
            "unique": True,
            "allowedDomains": [
                "mavq.com"
            ]
        }
    }
}

FIELD_UPDATE = {
    "kind": "ObjectField",
    "apiVersion": "objects.mavq.io/v1",
    "metadata": {
        "system": {
            "controls": {
                "access": {
                    "readOnly": False,
                    "hidden": False,
                    "nonDeletable": False
                }
            },
            "hooks": [],
            "audit": {
                "createdAt": "2022-06-07T07:02:47.713123384Z",
                "updatedAt": "2022-06-07T07:02:47.713123546Z",
                "createdBy": "mavq:core://tenants/admin/users/vishal-agarwal-back",
                "updatedBy": "mavq:core://tenants/admin/users/vishal-agarwal-back",
                "versions": [],
                "operations": [],
                "owner": "mavq:core://tenants/admin/users/vishal-agarwal-back",
                "versionNumber": 1
            },
            "selfLink": "mavq:objects://tenants/admin/object_fields/abcprojects-emailfield"
        },
        "tenant": "admin",
        "uid": "abcprojects-emailfield"
    },
    "spec": {
        "objectConfigurationUri": "mavq:objects://tenants/admin/object_configurations/abcprojects",
        "display": {
            "description": "",
            "helpText": "",
            "label": "emailfieldupdated",
            "placeholder": ""
        },
        "definition": {
            "type": "EMAIL",
            "name": "emailfieldupdated",
            "mandatory": True,
            "indexing": False,
            "immutable": False,
            "unique": True,
            "allowedDomains": [
                "mavq.com"
            ]
        }
    }
}

FIELD_SEARCH_QUERY = {
    "config": {
        "type": "PAGE",
        "count": True
    },
    "matchers": {
        "equality": {
            "spec.objectConfigurationUri": "mavq:objects://tenants/admin/object_configurations/{objectId}",
            "metadata.system.controls.access.hidden": False
        },
        "columns": [],
        "from": []
    }
}

LAYOUT_CREATE = {
    "kind": "ObjectLayoutV2",
    "apiVersion": "objects.mavq.io/v1",
    "metadata": {
        "tenant": "admin"
    },
    "spec": {
        "name": "pythonlayout",
        "description": "",
        "primaryObject": "mavq:objects://tenants/admin/object_configurations/abcprojects",
        "layoutContainers": {
            "quickCreateLayout": {
                "apiLayout": {
                    "objectName": "abcprojects",
                    "relatedField": None,
                    "fields": [
                        "name",
                        "id"
                    ],
                    "relatedObjects": [
                        {
                            "objectName": "users",
                            "relatedField": "owner",
                            "fields": [
                                "name"
                            ],
                            "relatedObjects": None
                        }
                    ]
                },
                "displayConfig": {},
                "validations": [],
                "buttons": []
            },
            "detailLayout": {
                "apiLayout": {
                    "objectName": "abcprojects",
                    "relatedField": None,
                    "fields": [
                        "name",
                        "id"
                    ],
                    "relatedObjects": [
                        {
                            "objectName": "users",
                            "relatedField": "owner",
                            "fields": [
                                "name"
                            ],
                            "relatedObjects": None
                        }
                    ]
                },
                "displayConfig": {},
                "validations": [],
                "buttons": []
            }
        }
    }
}

LAYOUT_UPDATE = {
    "kind": "ObjectLayoutV2",
    "apiVersion": "objects.mavq.io/v1",
    "metadata": {
        "tenant": "admin"
    },
    "spec": {
        "name": "pythonlayoutUpdated",
        "description": "hello",
        "primaryObject": "mavq:objects://tenants/admin/object_configurations/abcprojects",
        "layoutContainers": {
            "quickCreateLayout": {
                "apiLayout": {
                    "objectName": "abcprojects",
                    "relatedField": None,
                    "fields": [
                        "name",
                        "id"
                    ],
                    "manyToManyFields": [],
                    "relatedObjects": [
                        {
                            "objectName": "users",
                            "relatedField": "owner",
                            "fields": [
                                "name"
                            ],
                            "manyToManyFields": [],
                            "relatedObjects": []
                        }
                    ]
                },
                "displayConfig": {},
                "validations": [],
                "buttons": []
            },
            "detailLayout": {
                "apiLayout": {
                    "objectName": "abcprojects",
                    "relatedField": None,
                    "fields": [
                        "name",
                        "id"
                    ],
                    "manyToManyFields": [],
                    "relatedObjects": [
                        {
                            "objectName": "users",
                            "relatedField": "owner",
                            "fields": [
                                "name"
                            ],
                            "manyToManyFields": [],
                            "relatedObjects": []
                        }
                    ]
                },
                "displayConfig": {},
                "validations": [],
                "buttons": []
            }
        }
    }
}

LAYOUT_SEARCH_QUERY = {}

RECORDTYPE_CREATE = {
    "kind": "ObjectRecordType",
    "apiVersion": "objects.mavq.io/v1",
    "metadata": {
        "tenant": "admin"
    },
    "spec": {
        "name": "pythonrectype",
        "description": "",
        "active": False,
        "layoutAssignments": [
            {
                "profileUri": "mavq:core://tenants/admin/profiles/administrator",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-newlayout12"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/demoprofile",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/dhanashri",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/dssdsdsds",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/gami",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-newlayout123"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/nag",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/profiledemo",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/ritu",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/sayanka",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/standard-user",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/vinodh",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/vishal",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            }
        ],
        "label": "pythonrectype",
        "objectConfigurationUri": "mavq:objects://tenants/admin/object_configurations/abcprojects"
    }
}

RECORDTYPE_UPDATE = {
    "kind": "ObjectRecordType",
    "apiVersion": "objects.mavq.io/v1",
    "metadata": {
        "system": {
            "controls": {
                "access": {
                    "readOnly": False,
                    "hidden": False,
                    "nonDeletable": False
                }
            },
            "hooks": [],
            "audit": {
                "createdAt": "2022-06-07T10:46:20.883Z",
                "updatedAt": "2022-06-07T10:46:20.883Z",
                "createdBy": "mavq:core://tenants/admin/users/vishal-agarwal-back",
                "updatedBy": "mavq:core://tenants/admin/users/vishal-agarwal-back",
                "versions": [],
                "operations": [],
                "owner": "mavq:core://tenants/admin/users/vishal-agarwal-back",
                "versionNumber": 1
            },
            "selfLink": "mavq:objects://tenants/admin/object_record_types/abcprojects-pythonrectype"
        },
        "tenant": "admin",
        "uid": "abcprojects-pythonrectype"
    },
    "spec": {
        "name": "pythonrecordtypeupdate",
        "description": "",
        "layoutAssignments": [
            {
                "profileUri": "mavq:core://tenants/admin/profiles/administrator",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-newlayout12"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/demoprofile",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/dhanashri",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/dssdsdsds",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/gami",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-newlayout123"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/nag",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/profiledemo",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/ritu",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/sayanka",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/standard-user",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/vinodh",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            },
            {
                "profileUri": "mavq:core://tenants/admin/profiles/vishal",
                "layoutUri": "mavq:objects://tenants/admin/object_layouts_v2/abcprojects-system"
            }
        ],
        "label": "pythonrecordtypeupdate",
        "objectConfigurationUri": "mavq:objects://tenants/admin/object_configurations/abcprojects"
    }
}

RECORDTYPE_SEARCH_QUERY = {
    "config": {
        "type": "PAGE",
        "count": True
    },
    "matchers": {
        "equality": {
            "spec.objectConfigurationUri": "mavq:objects://tenants/admin/object_configurations/abcprojects",
            "metadata.system.controls.access.hidden": False
        },
        "columns": [],
        "from": []
    }
}

RECORD_CREATE = {
    "complaints": {
        "objectName": "complaints",
        "fields": {
            "id": None,
            "name": "python",
            "createdAt": None,
            "updatedAt": None,
            "recordType": "system",
            "status": "active",
            "email": "vishal.agarwal@mavq.com"
        },
        "relatedObject": [],
        "manyManyItems": {},
        "mode": "CREATE"
    }
}

RECORD_UPDATE = {
    "complaints": {
        "objectName": "complaints",
        "fields": {
            "owner": None,
            "name": "python",
            "meta": None,
            "id": "{recordId}",
            "recordType": None,
            "updatedBy": None,
            "status": "active",
            "createdBy": None,
            "createdAt": None,
            "updatedAt": None,
            "email": "vishal.agarwal@mavq.com"
        },
        "relatedObject": [],
        "manyManyItems": {},
        "mode": "UPDATE"
    }
}

RECORD_SEARCH_QUERY = {
    "type": "OQL",
    "oql": {
        "config": {
            "type": "RAW_VIA_LAYOUT",
            "count": True,
            "pagination": {
                "limit": 10,
                "skip": 0
            }
        },
        "query": {
            "columns": [],
            "from": [],
            "where": None,
            "orderby": [
                {
                    "expr": {
                        "type": "column_ref",
                        "column": "updatedAt",
                        "table": "complaints",
                        "alias": "complaints"
                    },
                    "type": "DESC"
                }
            ]
        },
        "apiLayout": {
            "objectName": "complaints",
            "relatedField": None,
            "fields": [
                "name",
                "updatedAt"
            ],
            "manyToManyFields": [],
            "relatedObjects": [
                {
                    "objectName": "users",
                    "relatedField": "updatedBy",
                    "fields": [
                        "name"
                    ],
                    "manyToManyFields": [],
                    "relatedObjects": []
                },
                {
                    "objectName": "users",
                    "relatedField": "owner",
                    "fields": [
                        "name"
                    ],
                    "manyToManyFields": [],
                    "relatedObjects": []
                }
            ]
        }
    }
}


import json
import requests
from commonProvider import URLENCODED_HEADER_ONLY
from mavqUserContext import MavQUserContext

"""
mavQ Platform client provide a support to login for your tenant user, and managing the
required configuration for client during the process.

@since 20-May-2022
@author Vishal Agarwal
@access public
"""
class mavqPlatformClient:

    """
    @param {Object} configuration - Configuration to init the MavQPlatformClient
    
    @example new MavQPlatformClient({
        tenant: 'system',
        coreApiBaseUrl: 'https://core.mavq.io',
        objectApiBaseUrl: 'https://objects.mavq.io',
        dataApiBaseUrl: 'https://data.mavq.io',
        authBaseUrl: 'https://auth.mavq.io',
        authClientId: 'login'
    })
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    """
    def __init__(self, tenant="invalid", coreApiBaseUrl="", objectApiBaseUrl="", dataApiBaseUrl="", authBaseUrl="", authClientId=""):
        self.tenant = tenant
        self.coreApiBaseUrl = coreApiBaseUrl
        self.objectApiBaseUrl = objectApiBaseUrl
        self.dataApiBaseUrl = dataApiBaseUrl
        self.authBaseUrl = authBaseUrl
        self.authClientId = authClientId

        """
        private variables specific to client
        """
        self.__DEFAULT_TENANT_NAME = "invalid"
        self.__DEFAULT_VALUE_TO_VALIDATE = [None, '']
        self.__GRANT_TYPE = 'password'
        self.__AUTH_SCOPE = 'openid'
        self.__access_token = None
    
    def version(self):
        return "0.0.2"

    """
    Validate the configuration for mavQPlatfomClient
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access private
    
    return String[] - If Any field does not contain valid value
    """
    def validateConfiguration(self):
        invalidFields = []
        if(self.tenant in [self.__DEFAULT_TENANT_NAME] or self.tenant in self.__DEFAULT_VALUE_TO_VALIDATE):
            invalidFields.append('tenant')
        if(self.coreApiBaseUrl in self.__DEFAULT_VALUE_TO_VALIDATE):
            invalidFields.append('coreApiBaseUrl')
        if(self.objectApiBaseUrl in self.__DEFAULT_VALUE_TO_VALIDATE):
            invalidFields.append('objectApiBaseUrl')
        if(self.dataApiBaseUrl in self.__DEFAULT_VALUE_TO_VALIDATE):
            invalidFields.append('dataApiBaseUrl')
        if(self.authBaseUrl in self.__DEFAULT_VALUE_TO_VALIDATE):
            invalidFields.append('authBaseUrl')
        if(self.authClientId in self.__DEFAULT_VALUE_TO_VALIDATE):
            invalidFields.append('authClientId')
        return invalidFields
    
    """
    Login and provide the user context for given tenant
    
    @param {String} userResourceUri - User resource uri
    @param {String} password - Password
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    return mavQUserContext - Return the mavQ User context
    """
    def loginAndGetUserContext(self, userResourceUri="", password=""):
        self.loginAndGetTokenOnly(userResourceUri, password)
        return MavQUserContext(self.__access_token, userResourceUri, password)
    
    """
    Login and provide the access token for given user
    
    @param {String} userResourceUri - User resource uri
    @param {String} password - Password
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    return {String} - Return the keycloak access token
    """
    def loginAndGetTokenOnly(self, userResourceUri="", password=""):
        
        invalidFields = self.validateConfiguration()
        if(invalidFields and len(invalidFields)):
            raise Exception("Missing or invalidate the required configuration properties {invalidFields}".format(invalidFields=", ".join(invalidFields)))
        
        params = {
            "client_id": self.authClientId,
            "username": userResourceUri,
            "password": password,
            "grant_type": self.__GRANT_TYPE
        }

        reqUrl = self.authBaseUrl + "/auth/realms/platform-auth-dev/protocol/openid-connect/token"

        response = requests.post(reqUrl, data=params, headers=URLENCODED_HEADER_ONLY)
        self.__access_token = json.loads(response.content)["access_token"]

        return response.content

    """
    Provide a request url of appropriate service of mavQ Platform
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access private
    
    @return {String} Return the service url for search operation on tenant and resource kind
    """
    def __getRequestUrl(self, service="", version=1, tenant="", operation="search/"):
        serviceUrl = ""
        if(service == "CORE"):
            serviceUrl = self.coreApiBaseUrl
        elif(service == "OBJECT"):
            serviceUrl = self.objectApiBaseUrl
        elif(service == "DATA"):
            serviceUrl = self.dataApiBaseUrl
        elif(service == "AUTH"):
            serviceUrl = self.authBaseUrl
        else:
            raise Exception("Invalid service request")
        reqUrl = serviceUrl + "/v{version}/tenants/{tenant}/{operation}".format(version=str(version), tenant=tenant, operation=operation)
        return reqUrl

    """
    Provide a List of Object fields for objectConfigurationUri
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the ObjectFields for objectConfigurationUri
    """
    def getObjectFields(self, searchQuery, searchId="search"):
        return self.__request("OBJECT", 1, "POST", searchQuery, ["object_fields", searchId])
    
    """
    Operation to create Object Field
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the ObjectField (created)
    """
    def createObjectField(self, body):
        return self.__request("OBJECT", 1, "POST", body, ["object_fields"])
    
    """
    Operation to update Object Field
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the ObjectField (updated)
    """
    def updateObjectField(self, body, fieldId):
        return self.__request("OBJECT", 1, "PUT", body, ["object_fields", fieldId])
    
    """
    Operation to delete Object Field
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return Status for deletion
    """
    def deleteObjectField(self, fieldId):
        return self.__request("OBJECT", 1, "DELETE", None, ["object_fields", fieldId])
    
    """
    Provide a List of Object Layouts for objectConfigurationUri
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the ObjectLayouts for objectConfigurationUri
    """
    def getObjectLayouts(self, searchQuery):
        return self.__request("OBJECT", 2, "POST", searchQuery, ["object_layouts"])
    
    """
    Operation to create Object Layout
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the ObjectLayout (created)
    """
    def createObjectLayout(self, body):
        return self.__request("OBJECT", 2, "POST", body, ["object_layouts"])
    
    """
    Operation to update Object Layout
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the ObjectLayout (updated)
    """
    def updateObjectLayout(self, body, layoutId):
        return self.__request("OBJECT", 2, "PUT", body, ["object_layouts", layoutId])

    """
    Provide a List of Object Record Types for objectConfigurationUri
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the ObjectRecordTypes for objectConfigurationUri
    """
    def getObjectRecordTypes(self, searchQuery, searchId="search"):
        if(searchId == "POST"):
            return self.__request("OBJECT", 1, "POST", searchQuery, ["object_record_types", searchId])
        return self.__request("OBJECT", 1, "GET", searchQuery, ["object_record_types", searchId])

    """
    Operation to create Object Layout
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the ObjectLayout (created)
    """
    def createObjectRecordType(self, body):
        return self.__request("OBJECT", 1, "POST", body, ["object_record_types"])
    
    """
    Operation to update Object Layout
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the ObjectLayout (updated)
    """
    def updateObjectRecordType(self, body, recordTypeId):
        return self.__request("OBJECT", 1, "PUT", body, ["object_record_types", recordTypeId])
    
    """
    Provide a List of Objects
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the Objects
    """
    def getObjects(self, searchQuery, objectName="search"):
        return self.__request("OBJECT", 1, "GET", searchQuery, ["object_configurations", objectName])
    
    """
    Operation to create record
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the created Object Record
    """
    def createObject(self, body):
        return self.__request("OBJECT", 1, "POST", body, ["operations"])
    
    """
    Operation to update record
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the updated Object Record
    """
    def updateObject(self, body, objectId):
        return self.__request("OBJECT", 1, "PUT", body, ["object_configurations", objectId])
    
    """
    Operation to delete record
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return confirmation status for operation
    """
    def deleteObject(self, objectId):
        return self.__request("OBJECT", 1, "DELETE", None, ["object_configurations", objectId])

    """
    Provide a List of Object Records with provided search request
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the Object Records
    """
    def searchObjectRecords(self, oqlQuery, searchId):
        return self.__request("OBJECT", 2, "POST", oqlQuery, ["object_manager", "sql", searchId])
    
    """
    Operation to create record
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the created Object Record
    """
    def createObjectRecord(self, body, objectName):
        return self.__request("OBJECT", 2, "POST", body, ["object_manager", "sql", objectName])
    
    """
    Operation to update record
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {JSON} Return the updated Object Record
    """
    def updateObjectRecord(self, body, objectName, recordId):
        return self.__request("OBJECT", 2, "PUT", body, ["object_manager", "sql", objectName, recordId])
    
    """
    Operation to delete record
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return confirmation status for operation
    """
    def deleteObjectRecord(self, objectName, recordId):
        return self.__request("OBJECT", 2, "DELETE", None, ["object_manager", "sql", objectName, recordId])

    """
    Operation to request BE Server
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access private
    
    @return {JSON} BE response as per request type
    """
    def __request(self, resourceType, apiVersion, requestType, bodyParams, queryParams):
        invalidFields = self.validateConfiguration()
        if(invalidFields and len(invalidFields)):
            raise Exception("Missing or invalidate the required configuration properties {invalidFields}".format(invalidFields=", ".join(invalidFields)))
        
        access_token = self.get_access_token()
        headers = {"Authorization": "Bearer {}".format(access_token), "Content-Type": "application/json"}

        queryParam =  "/".join([("" if x is None else x) for x in queryParams])
        reqUrl = self.__getRequestUrl(resourceType, apiVersion, self.tenant, queryParam) + "/"

        if(requestType == "POST"):
            return requests.post(reqUrl, json=bodyParams, headers=headers)
        elif(requestType == "PUT"):
            return requests.put(reqUrl, json=bodyParams, headers=headers)
        elif(requestType == "DELETE"):
            return requests.delete(reqUrl, headers=headers)
        elif(requestType == "GET"):
            return requests.get(reqUrl, headers=headers)
        else:
            raise Exception("Unknown request type")


    def get_access_token(self):
        return self.__access_token
        
    
    """
    Provide a service url of core service of mavQ Platform
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {String} Return the core service url
    """
    def getCoreServiceBaseUrl(self):
        return self.coreApiBaseUrl

    """
    Provide a service url of data service of mavQ Platform
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {String} Return the data service url
    """
    def getDataServiceBaseUrl(self):
        return self.dataApiBaseUrl

    """
    Provide a service url of objects service of mavQ Platform
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {String} Return the objects service url
    """
    def getObjectServiceBaseUrl(self):
        return self.objectApiBaseUrl
    
    """
    Provide a service url of Keycloak service of mavQ Platform
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {String} Return the Keycloak service url
    """
    def getAuthBaseUrl(self):
        return self.authBaseUrl
    
    """
    Provide a configured tenant name
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {String} Return the configured tenant name
    """
    def getTenantName(self):
        return self.tenant
    


from automationVariables import OBJECT_CREATE, OBJECT_UPDATE, FIELD_CREATE, FIELD_UPDATE, FIELD_SEARCH_QUERY, LAYOUT_CREATE, LAYOUT_UPDATE, LAYOUT_SEARCH_QUERY, RECORDTYPE_CREATE, RECORDTYPE_UPDATE, RECORDTYPE_SEARCH_QUERY, RECORD_CREATE, RECORD_UPDATE, RECORD_SEARCH_QUERY
from mavqPlatformClient import mavqPlatformClient

class AutomationClient:

    def __init__(self, mavqClient, objectId="pythonObject", objectName="complaints", fieldId="abcprojects-emailfield", layoutId="abcprojects-pythonlayout", recordTypeId = "abcprojects-pythonrectype", recordId = "ue1042001-8341-4201-9d54-e466f9df9dd7"):
        self.__objectId = objectId
        self.__objectName = objectName
        self.__fieldId = fieldId
        self.__layoutId = layoutId
        self.__recordId = recordId
        self.__mavqClient = mavqClient
        self.__recordTypeId = recordTypeId

    def __createObject(self):
        print(self.__mavqClient.createObject(OBJECT_CREATE).content)
    
    def __updateObject(self):
        print(self.__mavqClient.updateObject(OBJECT_UPDATE, self.__objectId).content)
    
    def __createField(self):
        print(self.__mavqClient.createObjectField(FIELD_CREATE).content)
    
    def __updateField(self):
        print(self.__mavqClient.updateObjectField(FIELD_UPDATE, self.__fieldId).content)
    
    def __getFields(self):
        print(self.__mavqClient.getObjectFields(FIELD_SEARCH_QUERY).content)
    
    def __deleteField(self):
        print(self.__mavqClient.deleteObjectField(self.__fieldId).content)
    
    def __createLayout(self):
        print(self.__mavqClient.createObjectLayout(LAYOUT_CREATE).content)
    
    def __updateLayout(self):
        print(self.__mavqClient.updateObjectLayout(LAYOUT_UPDATE, self.__layoutId).content)

    def __getLayouts(self):
        print(self.__mavqClient.getObjectLayouts(LAYOUT_SEARCH_QUERY).content)
    
    def __createRecordType(self):
        print(self.__mavqClient.createObjectRecordType(RECORDTYPE_CREATE).content)
    
    def __updateRecordType(self):
        print(self.__mavqClient.updateObjectRecordType(RECORDTYPE_UPDATE, self.__recordTypeId).content)
    
    def __getRecordTypes(self):
        print(self.__mavqClient.getObjectRecordTypes(RECORDTYPE_SEARCH_QUERY).content)
    
    def __createRecord(self):
        print(self.__mavqClient.createObjectRecord(RECORD_CREATE, self.__objectName).content)

    def __updateRecord(self):
        print(self.__mavqClient.updateObjectRecord(RECORD_UPDATE, self.__objectName, self.__recordId).content)
    
    def __searchRecords(self):
        print(self.__mavqClient.searchObjectRecords(RECORD_SEARCH_QUERY, self.__objectName).content)
    
    def __deleteRecord(self):
        print(self.__mavqClient.deleteObjectRecord(self.__objectName, self.__recordId).content)
    
    def __deleteObject(self):
        print(self.__mavqClient.deleteObject(self.__objectId).content)
    
    def automate(self):
        print("Creating Object --------------------------------")
        self.__createObject()
        print("Updating Object --------------------------------")
        self.__updateObject()
        print("Creating Field --------------------------------")
        self.__createField()
        print("Updating Field --------------------------------")
        self.__updateField()
        print("Fetching Fields --------------------------------")
        self.__getFields()
        print("Delete Field --------------------------------")
        self.__deleteField()
        print("Creating Layout --------------------------------")
        self.__createLayout()
        print("Updating Layout --------------------------------")
        self.__updateLayout()
        print("Fetch Layouts --------------------------------")
        self.__getLayouts()
        print("Creating Record Type --------------------------------")
        self.__createRecordType()
        print("Updating Record Type --------------------------------")
        self.__updateRecordType()
        print("Fetching Record Type --------------------------------")
        self.__getRecordTypes()
        print("Creating Record --------------------------------")
        self.__createRecord()
        print("Updating Record --------------------------------")
        self.__updateRecord()
        print("Fetch Records --------------------------------")
        self.__searchRecords()
        print("Delete Record --------------------------------")
        self.__deleteRecord()
        print("Delete Object --------------------------------")
        self.__deleteObject()

mavqClient = mavqPlatformClient("admin", "https://core-api-dev.internal.mavq.io", "https://objects-api-dev.internal.mavq.io", "https://data.mavq.io", "https://auth.mavq.com", "login")
mavqClient.loginAndGetTokenOnly("mavq:core://tenants/admin/users/vishal-agarwal-back", "sG1t65RBc5")
automationClient = AutomationClient(mavqClient)
automationClient.automate()



from pyrsistent import freeze

"""
Provide the base header like `content-type, accept`

@since 20-May-2022
@author Vishal Agarwal
@access public

@return {Object} - Base header
"""
BASE_HEADER_WITHOUT_AUTH = {
  'Content-Type': 'application/json',
  'Accept': 'application/json'
}

"""
Provide the base header like `content-type, accept` with Auth header `Authorization`

@since 20-May-2022
@author Vishal Agarwal
@access public

@return {Object} - Base header + `Authorization`
"""
def baseHeaderWithAuthToken(tokenWithoutBearer):
  return {'Authorization': 'Bearer {tokenWithoutBearer}'.format(tokenWithoutBearer=tokenWithoutBearer)} + BASE_HEADER_WITHOUT_AUTH


"""
Provide the `content-type: application/x-www-form-urlencoded` header

@since 20-May-2022
@author Vishal Agarwal
@access public

@return {Object} - `application/x-www-form-urlencoded` header only
"""
URLENCODED_HEADER_ONLY = {
  'Content-Type': 'application/x-www-form-urlencoded'
}

"""
Provide the constants of service name supported by mavQ Platform

@since 20-May-2022
@author Vishal Agarwal
@access public

@readonly

@return {Object} - Platform services constant
"""
PLATFORM_SERVICES = freeze({
  'CORE': 'CORE_SERVICE',
  'OBJECTS': 'OBJECTS_SERVICE',
  'DATA': 'DATA_SERVICE'
})

"""
Provide the constants of Object Type supported by mavQ Platform

@since 20-May-2022
@author Vishal Agarwal
@access public

@readonly

@return {Object} - Object Type constant
"""
OBJECT_TYPE = freeze({
  'SQL': 'sql',
  'NOSQL': 'nosql'
})


from tomlkit import datetime
import time
from commonProvider import OBJECT_TYPE, PLATFORM_SERVICES
import jwt

"""
MavQUserContext provide the context with supported various functionality to login in user

@since 20-May-2022
@author Vishal Agarwal
@access public
"""
class MavQUserContext:

    """
    @param {String} accessToken - Access token
    @param {String} userUri - User Resource Uri
    @param {String} password - Using just for regenerate the Access token if expire.
    @param {MavQPlatformClient} clientContext - Client context
    
    @example new MavQUserContext('TOKEN', 'mavq:core://tenants/admin/users/system-administrator', 'xyz', clientContext)
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    """
    def __init__(self, clientContext, accessToken = '', userUri = '', password = ''):
        self.accessToken = accessToken
        self.clientContext = clientContext
        self.userUri = userUri
        self.userPass = password

    """
    Provide a support to perform the any Request to mavQ Platform
    @param {String} requestMethod - Request Method like `get, post` etc
    @param {String} requestUrl - Request Url
    @param {Object} requestBody - Request body
    @param {Object} additionalHeaders - Additional headers apart from default one. Also override the default by it.
    @param {Object} requestQuerystring - Query params
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return Response of request
    """
    def performRequest(self, requestMethod, requestUrl = '', requestBody = {}, additionalHeaders = {}, requestQuerystring = {}):
        self.validateAndReassignAccessToken()
        return {
            "method": requestMethod,
            "url": requestUrl,
            "headers": {"Authorization": "Bearer {}".format(self.accessToken), "Content-Type": "application/json"} + additionalHeaders,
            "data": requestBody,
            "params": requestQuerystring
        }
  
    """
    Provide a support to perform the operation to mavQ Platform

    @param {PLATFORM_SERVICES} service - MavQ Platform Service like `CORE, DATA` etc
    @param {Object} operationData - Request body for operation
    @param {Object} additionalHeaders - Additional headers apart from default one. Also override the default by it.
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return Response of request
    """
    def performOperation(self, service = PLATFORM_SERVICES.CORE, operationData = {}, additionalHeaders = {}):
        operationUrl = 'v1/tenants/' + self.clientContext.getTenantName() + '/operations/'
        requestUrl = self.getBaseUrlByService(service)
        if(requestUrl.endswith('/')):
            requestUrl += operationUrl
        else:
            requestUrl = requestUrl + '/' + operationUrl
        return self.performRequest('post', requestUrl, operationData, additionalHeaders, {})
    
    """
    
    Provide a service url of mavQ Platform

    @param {String} service - MavQ Platform Service like `CORE, DATA` etc
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access public
    
    @return {String} Return the service url
    """
    def getBaseUrlByService(self, service):
        if(service.upper() == PLATFORM_SERVICES.CORE):
            return self.clientContext.getCoreServiceBaseUrl()
        elif(service.upper() == PLATFORM_SERVICES.DATA):
            return self.clientContext.getDataServiceBaseUrl()
        elif(service.upper() == PLATFORM_SERVICES.OBJECTS):
            return self.clientContext.getObjectServiceBaseUrl()
        else:
            raise Exception("Platform does not support given {service}, Supported services {services}".format(service=service.upper(), services=', '.join(PLATFORM_SERVICES.keys())))
    
    """
    Validate that given access token expire or not, If it's expire or about to expire then generate new token
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access private
    """
    def validateAndReassignAccessToken(self):
        tokenData = jwt.decode(self.accessToken)
        minuteDiff = datetime.datetime.fromtimestamp(tokenData.exp * 1000).getTime() - time.time()//60000
        if (minuteDiff <= 30):
            self.accessToken = self.clientContext.loginAndGetTokenOnly(self.userUri, self.userPass)

    """
    Create the object record on object service
    
    @param {OBJECT_TYPE} type - Object type like SQL, NOSQL
    @param {String} objectConfigurationId - Object Configuration Id
    @param {Object} requestBody - Request Body
    @param {Object} additionalHeader - Additional Header
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access Public
    
    @return {Axios<Response>} Response of request
    """
    def createRecord(self, objectConfigurationId, objectType = OBJECT_TYPE.SQL, requestBody = {}, additionalHeader = {}):
        return self.performRequest(
            'post',
            self.prepareObjectRecordUrlByType(objectType, objectConfigurationId),
            requestBody,
            additionalHeader
        )
    
    
    """
    Search the object record on object service

    @param {OBJECT_TYPE} type - Object type like SQL, NOSQL
    @param {String} objectConfigurationId - Object Configuration Id
    @param {Object} requestBody - Request Body
    @param {Object} additionalHeader - Additional Header
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access Public
    
    @return Response of request
    """
    def searchRecords(self, objectConfigurationId, objectType = OBJECT_TYPE.SQL, requestBody = {}, additionalHeader = {}):
        return self.performRequest(
            'post',
            self.prepareObjectRecordUrlByType(objectType, objectConfigurationId) + 'search/',
            requestBody,
            additionalHeader
        )
    
    """
    Get Specific object record from object service
    
    @param {OBJECT_TYPE} type - Object type like SQL, NOSQL
    @param {String} objectConfigurationId - Object Configuration Id
    @param {String} objectRecordId - Object Record Id
    @param {Object} additionalHeader - Additional Header
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access Public
    
    @return  Response of request
    """
    def getRecordById(self, objectConfigurationId, objectType = OBJECT_TYPE.SQL, objectRecordId = '', additionalHeader = {}):
        return self.performRequest(
            'get',
            self.prepareObjectRecordUrlByType(objectType, objectConfigurationId) + str(objectRecordId) + "/",
            {},
            additionalHeader
        )
    
    """
    Update Specific record on object service
    
    @param {OBJECT_TYPE} type - Object type like SQL, NOSQL
    @param {String} objectConfigurationId - Object Configuration Id
    @param {String} objectRecordId - Object Record Id
    @param {Object} requestBody - Request body
    @param {Object} additionalHeader - Additional Header
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access Public
    
    @return {Axios<Response>} Response of request
    """
    def updateRecordById(self, objectConfigurationId, objectType = OBJECT_TYPE.SQL, objectRecordId = '', requestBody ={}, additionalHeader = {}):
        return self.performRequest(
            'put',
            self.prepareObjectRecordUrlByType(objectType, objectConfigurationId) + str(objectRecordId) + "/",
            requestBody,
            additionalHeader
        )
    
    """
    Delete Specific object record from object service
    
    @param {OBJECT_TYPE} type - Object type like SQL, NOSQL
    @param {String} objectConfigurationId - Object Configuration Id
    @param {String} objectRecordId - Object Record Id
    @param {Object} additionalHeader - Additional Header
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access Public
    
    @return {Axios<Response>} Response of request
    """
    def deleteRecordById(self, objectConfigurationId, objectType = OBJECT_TYPE.SQL, objectRecordId = '', additionalHeader = {}):
        return self.performRequest(
            'delete',
            self.prepareObjectRecordUrlByType(objectType, objectConfigurationId) + str(objectRecordId) + '/',
            {},
            additionalHeader
        )
    
    """
    Prepare Object record url by object type
    
    @param {OBJECT_TYPE} type - Object type like SQL, NOSQL
    @param {String} objectConfigurationId - Object configuration Id
    
    @since 20-May-2022
    @author Vishal Agarwal
    @access Private
    
    @return {String} Return the service url
    """
    def prepareObjectRecordUrlByType(self, objectType = OBJECT_TYPE.SQL, objectConfigurationId = ''):
        requestUrl = self.clientContext.getObjectServiceBaseUrl()
        if (not requestUrl.endsWith('/')):
            requestUrl += '/'
        return '{requestUrl}v1/tenants/{tenant}/object_manager/{objectType}/{objectConfigurationId}/'.format(requestUrl=requestUrl, tenant=self.clientContext.getTenantName(), objectType=objectType, objectConfigurationId=objectConfigurationId)
